﻿/*
   This source code is offered under the Microsoft Public License (Ms-PL) which is outlined as follows:

   Microsoft Public License (Ms-PL)
   This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.

   1. Definitions
   The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
   A "contribution" is the original software, or any additions or changes to the software.
   A "contributor" is any person that distributes its contribution under this license.
   "Licensed patents" are a contributor's patent claims that read directly on its contribution.

   2. Grant of Rights
   (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
   (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.

   3. Conditions and Limitations
   (A) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
   (B) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
   (C) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
   (D) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
   (E) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
 */
using System;
using System.Collections;
using System.Linq.Expressions;
using System.Reflection;
using System.Diagnostics.CodeAnalysis;

namespace RenewalProjects.Libraries.Common.Testing.Helpers
{
    /// <summary>
    /// A basic way to get some reflection info from a linq expression.
    /// Note it only supports arguments that are constants.
    /// </summary>
    public static class BasicExpressionHelper<TContract>
    {
        /// <summary>
        /// Get method name and argument values from function
        /// </summary>
        [SuppressMessage("Microsoft.Design", "CA1011:ConsiderPassingBaseTypesAsParameters")]
        [SuppressMessage("Microsoft.Design", "CA1006:DoNotNestGenericTypesInMemberSignatures")]
		[SuppressMessage("Microsoft.Design", "CA1000:DoNotDeclareStaticMembersOnGenericTypes")]        
        public static MethodExpressionResult ObtainMethodAndArguments(Expression<Func<TContract>> expression)
		{
			MethodCallExpression methodCallExpr = (MethodCallExpression)expression.Body;
			return ObtainMethodAndArgumentsCommon(methodCallExpr);
		}

        /// <summary>
        /// Get method name and argument values from action
        /// </summary>
        [SuppressMessage("Microsoft.Design", "CA1011:ConsiderPassingBaseTypesAsParameters")]
        [SuppressMessage("Microsoft.Design", "CA1006:DoNotNestGenericTypesInMemberSignatures")]
        [SuppressMessage("Microsoft.Design", "CA1000:DoNotDeclareStaticMembersOnGenericTypes")]
		public static MethodExpressionResult ObtainMethodAndArguments(Expression<Action<TContract>> expression)
		{
			MethodCallExpression methodCallExpr = (MethodCallExpression)expression.Body;
			return ObtainMethodAndArgumentsCommon(methodCallExpr);
		}

		private static MethodExpressionResult ObtainMethodAndArgumentsCommon(MethodCallExpression methodCallExpr)
		{
		    MethodExpressionResult result = new MethodExpressionResult();
			result.Method = methodCallExpr.Method;
			
			ArrayList argumentList = new ArrayList();
			foreach (Expression argExpr in methodCallExpr.Arguments)
			{
				if (argExpr.NodeType == ExpressionType.Constant)
				{
					ConstantExpression e = (ConstantExpression)argExpr;
					argumentList.Add(e.Value);
				}
				else
				{
					throw new NotSupportedException(
						"Given expression contains operations/items that are not supported");
				}
			}
			
			result.Arguments = argumentList.ToArray();
			
			return result;
		}
    }
    
    /// <summary>
    /// Method results
    /// </summary>
    public sealed class MethodExpressionResult
    {
        /// <summary>
        /// Reflection Method
        /// </summary>
        public MethodInfo Method { get; internal set; }
        
        /// <summary>
        /// Arguments extracted from expression
        /// </summary>
		[SuppressMessage("Microsoft.Performance", "CA1819:PropertiesShouldNotReturnArrays")]
        public object[] Arguments { get; internal set; }
    }
}
