﻿/*
   This source code is offered under the Microsoft Public License (Ms-PL) which is outlined as follows:

   Microsoft Public License (Ms-PL)
   This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.

   1. Definitions
   The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
   A "contribution" is the original software, or any additions or changes to the software.
   A "contributor" is any person that distributes its contribution under this license.
   "Licensed patents" are a contributor's patent claims that read directly on its contribution.

   2. Grant of Rights
   (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
   (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.

   3. Conditions and Limitations
   (A) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
   (B) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
   (C) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
   (D) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
   (E) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
 */
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;

namespace WcfLibTest.Helpers
{
	public class ScientificComparer
	{
		public Action TestSet1 { get; set; }
		public Action TestSet2 { get; set; }

		public ScientificComparisonResults PerformTests(int repeatCount)
		{
			ScientificComparisonResults results = new ScientificComparisonResults();
			TimeSpan ave;
			TimeSpan total;

			PerformTests(repeatCount, TestSet1, out ave, out total);
			results.TestSet1Average = ave;
			results.TestSet1Total = total;

			PerformTests(repeatCount, TestSet2, out ave, out total);
			results.TestSet2Average = ave;
			results.TestSet2Total = total;

			return results;
		}

		private static void PerformTests(int repeatCount, Action testSet, out TimeSpan ave, out TimeSpan total)
		{
			List<long> results = new List<long>();

			Stopwatch sw = new Stopwatch();
			for (int i = 0; i < repeatCount; i++)
			{
				sw.Restart();

				testSet();

				sw.Stop();
				results.Add(sw.Elapsed.Ticks);
			}

			long aveTicks = (long) results.Average(s => s);
			ave = new TimeSpan(aveTicks);
			total = new TimeSpan(results.Sum(s => s));
		}
	}

	public class ScientificComparisonResults
	{
		public TimeSpan TestSet1Average { get; set; }
		public TimeSpan TestSet2Average { get; set; }
		public TimeSpan TestSet1Total { get; set; }
		public TimeSpan TestSet2Total { get; set; }
	}
}
