﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;

using WcfLibTest.Tests.Helpers;

namespace WcfLibTest
{
	class Program
	{
		public static void Main(string[] args)
		{
			TestBase[] testers = GetTesters();

			int pos = 1;

			Console.WriteLine("1. All tests");

			foreach (TestBase tester in testers)
			{
				pos++;
				Console.WriteLine("{0}. {1}", pos, tester.GetDescription());
			}

			Console.WriteLine("Select any of the given options and press enter.");
			string selection = Console.ReadLine();

			int option = InterpretSelection(selection);

			RunSelection(option, testers);

			Console.Write("Press any key to continue . . . ");
			Console.ReadKey(true);
		}

		private static void RunSelection(int option, TestBase[] testers)
		{
			if (option == 1)
			{
				Console.WriteLine("Performing all tests...");
				foreach (TestBase tester in testers)
				{
					Console.WriteLine("Running '{0}'...", tester.GetDescription());
					RunTests(tester.RunTests());
				}
			}
			else if (option > 1 && option <= testers.Length + 1)
			{
				Console.WriteLine("Performing tests for option {0}...", option);
				RunTests(testers[option - 2].RunTests());
			}
			else
			{
				Console.WriteLine("Invalid option");
			}
		}

		private static void RunTests(IEnumerable<TestResult> tests)
		{
			Console.WriteLine("Running tests...");
			foreach (TestResult result in tests)
			{
				if (result.Success)
				{
					Console.Write("'{0}' - {1} - ", result.Name, result.Elapsed);
					Console.ForegroundColor = ConsoleColor.Green;
					Console.WriteLine("Success");
					Console.ForegroundColor = ConsoleColor.Gray;
				}
				else
				{
					Console.Write("'{0}' - {1} - ", result.Name, result.Elapsed);
					Console.ForegroundColor = ConsoleColor.Red;
					Console.Write("Failure");
					Console.ForegroundColor = ConsoleColor.Gray;
					Console.Write(" - ");
					Console.ForegroundColor = ConsoleColor.Yellow;
					Console.WriteLine(result.Exception);
					Console.ForegroundColor = ConsoleColor.Gray;
				}
			}
		}

		private static int InterpretSelection(string selection)
		{
			int option;
			if (!Int32.TryParse(selection, out option))
			{
				return -1;
			}

			return option;
		}

		private static TestBase[] GetTesters()
		{
			Assembly current = Assembly.GetAssembly(typeof(Program));
			return current.GetTypes()
			       .Where(p => typeof(TestBase).Equals(p.BaseType))
			       .Select(s => (TestBase)Activator.CreateInstance(s))
			       .ToArray();
		}
	}
}