﻿
using System;
using System.ServiceModel;
using System.Threading;

using RenewalProjects.Libraries.Wcf.EasyProxy.Abstraction;
using RenewalProjects.Libraries.Wcf.EasyProxy.ChannelAllocationStrategies;
using RenewalProjects.Libraries.Wcf.EasyProxy.ChannelAllocationStrategies.Settings;
using RenewalProjects.Libraries.Wcf.EasyProxy.ServiceAccessConsumers;
using WcfLibTest.Tests.Helpers;

namespace WcfLibTest.Tests
{
	public class BasicChannelPoolWithWSHttpTester : TestBase
	{
		#region Fields

		private ServiceHost host;
		private const string ADDRESS = "http://localhost:9009/test";
		private IServiceAccess<ITestService> access;

		#endregion

		#region TestBase

		public override string GetDescription()
		{
			return "Basic Channel Pool tests with WSHttp";
		}

		protected override void TestBegin()
		{
			ChannelRepository<ITestService> cr =
			    new ChannelRepository<ITestService>(
			        GetBinding(), new EndpointAddress(ADDRESS));

			PoolChannelAllocationSettings settings = new PoolChannelAllocationSettings();
			settings.ReadyChannelMax = 5;
			settings.StandbyChannelMax = 5;

			PooledChannelAllocationStrategy<ITestService> pool =
			    new PooledChannelAllocationStrategy<ITestService>();

			host = new ServiceHost(typeof(TestService));
			host.AddServiceEndpoint(typeof(ITestService), GetBinding(), ADDRESS);
			host.Open();

			cr.ChannelAllocationStrategy = pool;
			pool.RefreshChannelPools();
			access = cr;
		}

		protected override void TestEnd()
		{
			host.Close();
		}

		private WSHttpBinding GetBinding()
		{
			WSHttpBinding binding = new WSHttpBinding();
			/*binding.Security.Mode = SecurityMode.Message;
			   binding.Security.Message.ClientCredentialType = MessageCredentialType.None;
			   binding.Security.Transport.ClientCredentialType = HttpClientCredentialType.None;*/

			return binding;
		}

		#endregion

		#region Tests

		public void TestWithFewConnections()
		{
			for (int i = 0; i < 10; i++)
			{
				ThreadGroup th =
				    new ThreadGroup(
				        () =>
				        {
				            access.AccessInterface(x => x.GetName());
						}, 10)
				    .Start()
				    .WaitForThreads();
			}
		}

		public void TestThatChannelsWorkAfterFirstFewFaults()
		{
			for (int i = 0; i < 20; i++)
			{
				ThreadGroup th =
				    new ThreadGroup(
				        () =>
				        {
				            try
				            {
				                access.AccessInterface(x => x.ThrowErrorFirstFewTimes());
							}
				            catch (FaultException)
				            {
				                Console.WriteLine("Fault caught");
							}
						}, 2)
				    .Start()
				    .WaitForThreads();
			}
		}

		#endregion

		#region Test Helpers

		[ServiceContract]
		public interface ITestService
		{
			[OperationContract]
			string GetName();

			[OperationContract]
			void ThrowErrorFirstFewTimes();
		}

		public class TestService : ITestService
		{
			public string GetName()
			{
				return "AAA";
			}

			private static volatile int _count;

			public void ThrowErrorFirstFewTimes()
			{
				_count++;
				if (_count < 10)
				{
					throw new Exception();
				}
			}
		}

		#endregion
	}
}
