﻿
using System;
using System.ServiceModel;
using RenewalProjects.Libraries.Wcf;
using RenewalProjects.Libraries.Wcf.EasyProxy.Abstraction;
using RenewalProjects.Libraries.Wcf.EasyProxy.ServiceAccessConsumers;
using WcfLibTest.Tests.Helpers;

namespace WcfLibTest.Tests
{
	public class BasicIntegrationTester : TestBase
	{
		#region Fields

		private ServiceHost host;
		private const string ADDRESS = "net.pipe://localhost/test";
		private IServiceAccess<ITestService> access = new ChannelRepository<ITestService>(
		    new NetNamedPipeBinding(), new EndpointAddress(ADDRESS));

		#endregion

		#region TestBase

		public override string GetDescription()
		{
			return "Test basic Wcf integration";
		}

		protected override void TestBegin()
		{
			host = new ServiceHost(typeof(TestService));
			host.AddServiceEndpoint(typeof(ITestService), new NetNamedPipeBinding(), ADDRESS);
			host.Open();
		}

		protected override void TestEnd()
		{
			host.Close();
		}

		#endregion

		#region Tests

		public void TestSimpleOperation()
		{
			Console.WriteLine(access.AccessInterface(x => x.GetName()));
		}

		public void TestValueRetrieval()
		{
			access.AccessInterface(x => x.SetFlag(true));
			Console.WriteLine(access.AccessInterface(x => x.GetFlagStatus()));
		}

		public void TestFaultReceive()
		{
			try
			{
				access.AccessInterface(x => x.ThrowSomeFault());
				throw new Exception("Should have caught fault exception");
			}
			catch (FaultException)
			{
			}
		}

		public void TestSuddenChannelClosing()
		{
			try
			{
				access.AccessInterface(x => x.CloseChannelOnMe());
				throw new Exception("Should have caught Comminucation exception");
			}
			catch (CommunicationException)
			{
			}
		}

		#endregion

		#region Test Helpers

		[ServiceContract]
		public interface ITestService
		{
			[OperationContract]
			string GetName();

			[OperationContract]
			void SetFlag(bool state);

			[OperationContract]
			bool GetFlagStatus();

			[OperationContract]
			void ThrowSomeFault();

			[OperationContract]
			void CloseChannelOnMe();
		}

		[ServiceBehavior(InstanceContextMode = InstanceContextMode.Single)]
		public class TestService : ITestService
		{
			public bool Flag { get; set; }

			public string GetName()
			{
				return "Mark";
			}

			public void SetFlag(bool state)
			{
				Flag = state;
			}

			public bool GetFlagStatus()
			{
				return Flag;
			}

			public void ThrowSomeFault()
			{
				throw new Exception("Here is your fault");
			}

			public void CloseChannelOnMe()
			{
				OperationContext.Current.Channel.Close();
			}
		}

		#endregion
	}
}
