﻿
using System;
using System.ServiceModel;
using RenewalProjects.Libraries.Wcf.EasyProxy.Abstraction;
using RenewalProjects.Libraries.Wcf.EasyProxy.ChannelAllocationStrategies;
using RenewalProjects.Libraries.Wcf.EasyProxy.ChannelAllocationStrategies.Settings;
using RenewalProjects.Libraries.Wcf.EasyProxy.ServiceAccessConsumers;
using WcfLibTest.Helpers;
using WcfLibTest.Tests.Helpers;

namespace WcfLibTest.Tests
{
	public class ChannelPoolVsNormalPoolWSHttpTester : TestBase
	{
		#region Fields

		private ServiceHost host;
		private const string ADDRESS = "http://localhost:9009/test";
		private IServiceAccess<ITestService> access;

		#endregion

		#region TestBase

		public override string GetDescription()
		{
			return "Channel Pool VS Normal Pool volume tests with WSHttp";
		}

		protected override void TestBegin()
		{
			ChannelRepository<ITestService> cr =
			    new ChannelRepository<ITestService>(
			        GetBinding(), new EndpointAddress(ADDRESS));

			PoolChannelAllocationSettings settings = new PoolChannelAllocationSettings();
			settings.ReadyChannelMax = 10;
			settings.StandbyChannelMax = 30;

			PooledChannelAllocationStrategy<ITestService> pool =
			    new PooledChannelAllocationStrategy<ITestService>();

			host = new ServiceHost(typeof(TestService));
			host.AddServiceEndpoint(typeof(ITestService), GetBinding(), ADDRESS);
			host.Open();

			cr.ChannelAllocationStrategy = pool;
			pool.RefreshChannelPools();
			access = cr;
		}

		protected override void TestEnd()
		{
			host.Close();
		}

		private WSHttpBinding GetBinding()
		{
			WSHttpBinding binding = new WSHttpBinding();

			return binding;
		}

		#endregion

		#region Tests

		public void TestChannelPoolWithNormalChannelWithWSHttp_10()
		{
			BasicTestMethod(10, 6);
		}

		public void TestChannelPoolWithNormalChannelWithWSHttp_50()
		{
			BasicTestMethod(50, 6);
		}

		public void TestChannelPoolWithNormalChannelWithWSHttp_150()
		{
			BasicTestMethod(150, 6);
		}

		#endregion

		#region Helper methods

		private void BasicTestMethod(int connections, int repeatCount)
		{
			Console.WriteLine("{0} connections each (repeated {1} times)", connections, repeatCount);
			
			ChannelRepository<ITestService> r =
			    new ChannelRepository<ITestService>(
			        GetBinding(), new EndpointAddress(ADDRESS));

			ScientificComparer comparer = new ScientificComparer();

			comparer.TestSet1 = () =>
			{
				ThreadGroup th =
				    new ThreadGroup(
				        () =>
				        {
				            access.AccessInterface(x => x.GetName());
						}, connections)
				    .Start()
				    .WaitForThreads();
			};
			comparer.TestSet2 = () =>
			{
				ThreadGroup th =
				    new ThreadGroup(
				        () =>
				        {
				            r.AccessInterface(x => x.GetName());
						}, connections)
				    .Start()
				    .WaitForThreads();
			};

			ScientificComparisonResults results = comparer.PerformTests(repeatCount);

			Console.WriteLine("Channel Pool average time : {0}", results.TestSet1Average);
			Console.WriteLine("Normal Channel average time : {0}", results.TestSet2Average);
			Console.WriteLine("Channel Pool total time : {0}", results.TestSet1Total);
			Console.WriteLine("Normal Channel total time : {0}", results.TestSet2Total);
		}

		#endregion

		#region Test Helpers

		[ServiceContract]
		public interface ITestService
		{
			[OperationContract]
			string GetName();
		}

		public class TestService : ITestService
		{
			public string GetName()
			{
				return "AAA";
			}
		}

		#endregion
	}
}
