﻿
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Reflection;

namespace WcfLibTest.Tests.Helpers
{
	public abstract class TestBase
	{
		public abstract string GetDescription();

		protected virtual void TestBegin()
		{
		}

		protected virtual void TestEnd()
		{
		}

		public IEnumerable<TestResult> RunTests()
		{
			MethodInfo[] methods = this.GetType()
			                       .GetMethods()
			                       .Where(p => p.Name.StartsWith("Test"))
			                       .ToArray();

			Stopwatch totalTimer = new Stopwatch();

			foreach (MethodInfo method in methods)
			{
				TestResult result;
				try
				{
					totalTimer.Restart();

					TestBegin();

					method.Invoke(this, null);

					TestEnd();

					totalTimer.Stop();

					result = new TestResult(method.Name, totalTimer.Elapsed, true, null);
				}
				catch (Exception ex)
				{
					totalTimer.Stop();

					result = new TestResult(method.Name, totalTimer.Elapsed, false, ex);
				}
				
				yield return result;
			}
		}
	}
}
