﻿
using System;
using System.ServiceModel;
using System.Threading;

using RenewalProjects.Libraries.Wcf.EasyProxy.Abstraction;
using RenewalProjects.Libraries.Wcf.EasyProxy.ChannelAllocationStrategies;
using RenewalProjects.Libraries.Wcf.EasyProxy.ChannelAllocationStrategies.Settings;
using RenewalProjects.Libraries.Wcf.EasyProxy.ServiceAccessConsumers;
using WcfLibTest.Tests.Helpers;

namespace WcfLibTest.Tests
{
	public class LongStandingPoolWithWSHttpTester : TestBase
	{
		#region Fields

		private ServiceHost host;
		private const string ADDRESS = "http://localhost:9009/test";
		private IServiceAccess<ITestService> access;

		#endregion

		#region TestBase

		public override string GetDescription()
		{
			return "Test Channel Pool long standing with WSHttp";
		}

		protected override void TestBegin()
		{
			ChannelRepository<ITestService> cr =
			    new ChannelRepository<ITestService>(
			        GetBinding(), new EndpointAddress(ADDRESS));

			PoolChannelAllocationSettings settings = new PoolChannelAllocationSettings();
			settings.ReadyChannelMax = 1;
			settings.StandbyChannelMax = 0;

			PooledChannelAllocationStrategy<ITestService> pool =
			    new PooledChannelAllocationStrategy<ITestService>();

			host = new ServiceHost(typeof(TestService));
			host.AddServiceEndpoint(typeof(ITestService), GetBinding(), ADDRESS);
			host.Open();

			cr.ChannelAllocationStrategy = pool;
			pool.RefreshChannelPools();
			access = cr;
		}

		protected override void TestEnd()
		{
			host.Close();
		}

		private WSHttpBinding GetBinding()
		{
			WSHttpBinding binding = new WSHttpBinding();

			return binding;
		}

		#endregion
		
		#region Tests

		public void TestWithLongStandingMessageSecurity()
		{
			access.AccessInterface(x => x.GetName());

			Thread.Sleep(30000);

			access.AccessInterface(x => x.GetName());
		}
		
		#endregion
		
		#region Test Helpers

		[ServiceContract]
		public interface ITestService
		{
			[OperationContract]
			string GetName();
		}

		public class TestService : ITestService
		{
			public string GetName()
			{
				return "AAA";
			}
		}

		#endregion
	}
}
