﻿using System;
using System.Reflection;
using System.ServiceModel;
using System.ServiceModel.Channels;
using NUnit.Framework;
using RenewalProjects.Libraries.Common.Testing.Attributes;
using RenewalProjects.Libraries.Common.Testing.Interfaces;
using RenewalProjects.Libraries.Wcf.EasyProxy.CallBehaviors;
using RenewalProjects.Libraries.Wcf.EasyProxy.ChannelAllocators;
using RenewalProjects.Libraries.Wcf.EasyProxy.ProxyFactories;
using RenewalProjects.Libraries.Wcf.EasyProxy.Tests.TestData;

namespace RenewalProjects.Libraries.Wcf.EasyProxy.Tests.Fixtures
{
	[ServiceHostMetadata(Service = typeof(TestService), Contract = typeof(ITest))]
	[TestFixture]
	public class EasyProxyIntegrationFixture : IUsesServiceHosts
	{
		#region Properties

		public ServiceHost Host { get; set; }

		public Binding Binding { get; set; }

		public string Address { get; set; }

		#endregion

		#region Test Methods

		[IntegrationTest]
		[Test]
		public void EasyProxyIntegration_simple_call_over_netpipe_to_get_name()
		{
			ChannelFactory<ITest> channelFactory = new ChannelFactory<ITest>(Binding, Address);
			TransientChannelAllocator<ITest> channelAllocator = 
			    new TransientChannelAllocator<ITest>(channelFactory, channelFactory.Endpoint);
			DefaultCallBehavior<ITest> callBehavior = new DefaultCallBehavior<ITest>(channelAllocator);
			
			var proxy = EasyProxyFactory.CreateProxy<ITest>(callBehavior);
			Assert.AreEqual("Test Service Name", proxy.GetName());
		}
		
		[IntegrationTest]
		[ExpectedException(typeof(TargetInvocationException))]
		[Test]
		public void EasyProxyIntegration_simple_call_over_netpipe_receive_exception()
		{
		    ChannelFactory<ITest> channelFactory = new ChannelFactory<ITest>(Binding, Address);
			TransientChannelAllocator<ITest> channelAllocator = 
			    new TransientChannelAllocator<ITest>(channelFactory, channelFactory.Endpoint);
			DefaultCallBehavior<ITest> callBehavior = new DefaultCallBehavior<ITest>(channelAllocator);
			
			var proxy = EasyProxyFactory.CreateProxy<ITest>(callBehavior);
			proxy.DoSomethingDodgy();
		}
		
		#endregion

		#region Helper Code

		public class TestService : ITest
		{
			public string GetName()
			{
				return "Test Service Name";
			}

			public void DoSomethingDodgy()
			{
				throw new NotImplementedException();
			}
		}

		#endregion
	}
}